/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.export;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.isobus.model.BinaryWriter;
import nl.cloudfarming.client.isobus.model.DDI;
import nl.cloudfarming.client.isobus.model.GridTypeEnum;
import nl.cloudfarming.client.isobus.type.GridFieldEnum;
import org.openide.filesystems.FileAlreadyLockedException;
import org.openide.filesystems.FileObject;

/**
 * Writer class for grid binary files Fires property change event
 * GridWriter.PROP_FINISHED when writing is finished
 *
 * @author Merijn Zengers
 */
public class GridWriter {

    private static final Logger LOGGER = Logger.getLogger(GridWriter.class.getName());

    /**
     * Writes a grid file to the supplied destination FileObject Registers a
     * FileListener on the file which listens to file change When the file is
     * changed it fires a propertchangeEvent PROP_FINISHED If this property is
     * not filled an assertion exception is thrown
     *
     * @param destination the destination folder this grid file should be
     * written in
     * @param gridType the type to write 
     * @param TreatmentZoneGrid the TreatmentZoneGrid containing the treatment
     * zones mapped to unique ids
     */
    public void write(FileObject destination, GridTypeEnum gridType, TreatmentZoneGrid treatmentZoneGrid, DDI ddi, double multiplier) {
        // Generate actual grid
        try {
            try (BinaryWriter binaryWriter = new BinaryWriter(destination.getOutputStream())) {
                for (int row = treatmentZoneGrid.getNumberOfRows() - 1; row >= 0; row--) {
                    for (TreatmentZoneGrid.TreatmentZoneData data : treatmentZoneGrid.getRow(row)) {
                        if (GridTypeEnum.TYPE_1.equals(gridType)) {
                            binaryWriter.write(GridFieldEnum.TREATMENT_ZONE_CODE_TYPE, data.getCode());
                        } else if(GridTypeEnum.TYPE_2.equals(gridType)){
                            binaryWriter.write(GridFieldEnum.TREATMENT_ZONE_VALUE_TYPE, ddi.toExportValue(data.getValue() * multiplier));
                        }
                    }
                }
            }
        } catch (FileAlreadyLockedException ex) {
            LOGGER.log(Level.SEVERE, "Failed writing gridfile. (File already locked) with exception {0}", ex.getMessage());
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Failed writing gridfile. (IOException) with exception", ex.getMessage());
        }
    }
}
